"""
Add a sound file to an offline version
"""


def register(parser):
    pass
    #
    # FIXME: language and parameter ID, sound file path, optionally transcription
    # -> ask Mary what the most likely scenario for adding data would be.
    #


def run(args):
    pass
    #
    # FIXME: must add to HTML as well!
    #
