<!doctype html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">

    <script src="data.js"></script>

    <link rel="stylesheet" href="{% if not index %}../{% endif %}static/leaflet.css"/>
    <link rel="stylesheet" href="{% if not index %}../{% endif %}static/leaflet.fullscreen.css"/>
    <link rel="stylesheet" href="{% if not index %}../{% endif %}static/bootstrap.min.css">
    <link rel="stylesheet" href="{% if not index %}../{% endif %}static/project.css">
    <script src="{% if not index %}../{% endif %}static/leaflet.js"></script>
    <script src="{% if not index %}../{% endif %}static/Leaflet.fullscreen.js"></script>
    <style>
        #map {
            height: 600px;
        }
        .scrollable-menu {
            height: auto;
            max-height: 500px;
            overflow-x: hidden;
        }
    </style>
</head>
<body>
<nav class="navbar navbar-expand-md navbar-dark bg-dark">
    <a class="navbar-brand" href="{% if not index %}../{% endif %}index.html" title="{{ title_tooltip }}">{{ title }}</a>
    <button class="navbar-toggler" type="button" data-toggle="collapse" data-target="#navbarsExampleDefault"
            aria-controls="navbarsExampleDefault" aria-expanded="false" aria-label="Toggle navigation">
        <span class="navbar-toggler-icon"></span>
    </button>

    <div class="collapse navbar-collapse" id="navbarsExampleDefault">
        <ul class="navbar-nav mr-auto">
            <!--li class="nav-item active">
                <a class="nav-link" href="#">Home <span class="sr-only">(current)</span></a>
            </li>
            <li class="nav-item">
                <a class="nav-link" href="#">Link</a>
            </li>
            <li class="nav-item">
                <a class="nav-link disabled" href="#" tabindex="-1" aria-disabled="true">Disabled</a>
            </li-->
            <li class="nav-item dropdown">
                <a class="nav-link dropdown-toggle" href="#" id="dropdown01" data-toggle="dropdown" aria-haspopup="true"
                   aria-expanded="false">Parameters</a>
                <div class="dropdown-menu scrollable-menu" aria-labelledby="dropdown01">
                    {% for pid, parameter in parameters %}
                        <a class="dropdown-item" href="{% if not index %}../{% endif %}parameter-{{ pid }}/index.html">{{ parameter['name'] }}</a>
                    {% endfor %}
                </div>
            </li>
        </ul>
    </div>
</nav>

<div class="container">
{% block content %}

{% endblock %}
</div><!-- /.container -->


<script src="{% if not index %}../{% endif %}static/jquery-3.5.1.min.js"></script>
<script src="{% if not index %}../{% endif %}static/bootstrap.bundle.min.js"></script>
<script src="{% if not index %}../{% endif %}static/offline.js"></script>

</body>
</html>
